@@ -19,6 +19,7 @@
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
+#include "config.h"
 #include "array.h"
 
 #include <stdlib.h>
@@ -55,7 +56,8 @@ static gchar* valid_key_map[] = {
     "8v", //,
     "9v", //.
     "0v", ///
-    "0-"  //;
+    "0-", //;
+    " ?", //?
 };
 
 ArrayContext* array_create_context() {
@@ -98,6 +100,9 @@ GString* array_get_preedit_string(GString *preedit) {
         else if (c == ';') {
             index = 29;
         }
+        else if (c == '?') {
+            index = 30;
+        }
 
         if (index >= 0) {
             g_string_append(result, valid_key_map[index]);
@@ -116,14 +121,40 @@ void array_release_candidates(GArray *candidates) {
     g_array_free(candidates, TRUE);
 }
 
-GArray* array_get_candidates_from_main(ArrayContext *context, gchar *keys) {
+GArray* array_get_candidates_from_main(ArrayContext *context, gchar *keys, guint wildcard_char_count) {
     GArray *result;
     result = (GArray*)g_array_new(FALSE, FALSE, sizeof(gchar*));
 
     sqlite3_stmt *stmt;
 
     int retcode;
-    retcode = sqlite3_prepare_v2(context->conn, "SELECT ch FROM main WHERE keys=?", -1, &stmt, NULL);
+    if (!wildcard_char_count)
+        retcode = sqlite3_prepare_v2(context->conn, "SELECT ch FROM main WHERE keys=?", -1, &stmt, NULL);
+    else
+        retcode = sqlite3_prepare_v2(context->conn, "SELECT ch FROM main WHERE keys GLOB ?", -1, &stmt, NULL);
+
+    if (retcode == SQLITE_OK) {
+        sqlite3_bind_text(stmt, 1, keys, -1, SQLITE_TRANSIENT);
+        while (sqlite3_step(stmt) == SQLITE_ROW) {
+            gchar *ch = (gchar*)sqlite3_column_text(stmt, 0);
+            gchar *chstr = g_strdup(ch);
+            g_array_append_val(result, chstr);
+        }
+    }
+    sqlite3_reset(stmt);
+    sqlite3_finalize(stmt);
+
+    return result;
+}
+
+GArray* array_get_candidates_from_phrase(ArrayContext *context, gchar *keys) {
+    GArray *result;
+    result = (GArray*)g_array_new(FALSE, FALSE, sizeof(gchar*));
+
+    sqlite3_stmt *stmt;
+
+    int retcode;
+    retcode = sqlite3_prepare_v2(context->conn, "SELECT ph FROM phrase WHERE keys=?", -1, &stmt, NULL);
     if (retcode == SQLITE_OK) {
         sqlite3_bind_text(stmt, 1, keys, -1, SQLITE_TRANSIENT);
         while (sqlite3_step(stmt) == SQLITE_ROW) {
@@ -226,8 +257,8 @@ GArray* array_get_reverted_char_candidates_from_special(ArrayContext *context, g
     return result;
 }
 
-gboolean 
-array_input_key_is_not_special(ArrayContext* context, 
+gboolean
+array_input_key_is_not_special(ArrayContext* context,
                                 const gchar* keys, const gchar* ch)
 {
     sqlite3_stmt *stmt;
@@ -236,8 +267,8 @@ array_input_key_is_not_special(ArrayContext* context,
     gboolean result = FALSE;
     gchar *special_keys = NULL;
 
-    retcode = sqlite3_prepare_v2(context->conn, 
-            "SELECT keys FROM special WHERE ch=?", -1, &stmt, NULL);
+    retcode = sqlite3_prepare_v2(context->conn,
+            "SELECT keys FROM main WHERE cat='2' AND ch=?", -1, &stmt, NULL);
 
     if (retcode == SQLITE_OK) {
         sqlite3_bind_text(stmt, 1, ch, -1, SQLITE_TRANSIENT);
