@@ -25,6 +25,10 @@
 #include "array.h"
 #include "config.h"
 
+#ifdef HAVE_OPENCC
+#  include <opencc.h>
+#endif
+
 #define _(String) gettext(String)
 #define ARRAY_SHORT_CODE_EMPTY_STRING "⎔"
 
@@ -38,6 +42,7 @@ struct _IBusArrayEngine {
     GString *preedit;
     gint cursor_pos;
     guint space_press_count;
+    guint wildcard_char_count;
 
     IBusLookupTable *table;
     IBusPropList *prop_list;
@@ -69,7 +74,7 @@ static void ibus_array_engine_page_down (IBusEngine *engine);
 static void ibus_array_engine_cursor_up (IBusEngine *engine);
 static void ibus_array_engine_cursor_down (IBusEngine *engine);
 
-static void ibus_array_engine_property_activate (IBusEngine *engine, const gchar *prop_name, gint prop_state);
+static void ibus_array_engine_property_activate (IBusEngine *engine, const gchar *prop_name, guint prop_state);
 static void ibus_array_engine_property_show (IBusEngine *engine, const gchar *prop_name);
 static void ibus_array_engine_property_hide (IBusEngine *engine, const gchar *prop_name);
 
@@ -84,6 +89,8 @@ static gboolean ibus_array_engine_commit_current_candidate (IBusArrayEngine *arr
 
 static void ibus_array_engine_space_press(IBusArrayEngine *arrayeng);
 
+static gboolean ibus_array_engine_update_phrase_lookup_table (IBusArrayEngine *arrayeng);
+
 static gboolean ibus_array_engine_update_symbol_lookup_table (IBusArrayEngine *arrayeng);
 
 static void ibus_array_engine_update_auxiliary_text (IBusArrayEngine *arrayeng, gchar* aux_string);
@@ -97,8 +104,27 @@ static IBusEngineClass *parent_class = NULL;
 static IBusConfig *config = NULL;
 static gboolean is_special_notify;
 static gboolean is_special_only;
+static gboolean is_output_simplified;
+static gboolean is_use_shift;
+static gboolean is_english_mode;
+static gboolean is_fullwidth_mode;
 static gboolean is_aux_shown = FALSE;
+static gint prev_pressed_key = IBUS_VoidSymbol;
 static ArrayContext *array_context = NULL;
+#ifdef HAVE_OPENCC
+static opencc_t cc_handle = NULL;;
+#endif
+
+static char *sFullWidthTable[] = {
+    "　", "！", "＂", "＃", "＄", "％", "＆", "＇", "（", "）", "＊", "＋",
+    "，", "－", "．", "／", "０", "１", "２", "３", "４", "５", "６", "７",
+    "８", "９", "：", "；", "＜", "＝", "＞", "？", "＠", "Ａ", "Ｂ", "Ｃ",
+    "Ｄ", "Ｅ", "Ｆ", "Ｇ", "Ｈ", "Ｉ", "Ｊ", "Ｋ", "Ｌ", "Ｍ", "Ｎ", "Ｏ",
+    "Ｐ", "Ｑ", "Ｒ", "Ｓ", "Ｔ", "Ｕ", "Ｖ", "Ｗ", "Ｘ", "Ｙ", "Ｚ", "［",
+    "＼", "］", "＾", "＿", "｀", "ａ", "ｂ", "ｃ", "ｄ", "ｅ", "ｆ", "ｇ",
+    "ｈ", "ｉ", "ｊ", "ｋ", "ｌ", "ｍ", "ｎ", "ｏ", "ｐ", "ｑ", "ｒ", "ｓ",
+    "ｔ", "ｕ", "ｖ", "ｗ", "ｘ", "ｙ", "ｚ", "｛", "｜", "｝", "～",
+};
 
 GType ibus_array_engine_get_type (void) {
 	static GType type = 0;
@@ -132,6 +158,10 @@ void ibus_array_init (IBusBus *bus) {
 
     is_special_notify = FALSE;
     is_special_only = FALSE;
+    is_output_simplified = FALSE;
+    is_use_shift = FALSE;
+    is_english_mode = FALSE;
+    is_fullwidth_mode = FALSE;
 
     /* load config */
     GVariant* value;
@@ -144,15 +174,40 @@ void ibus_array_init (IBusBus *bus) {
     if (value && g_variant_classify(value) == G_VARIANT_CLASS_BOOLEAN)
             is_special_only = g_variant_get_boolean(value);
 
+    value = ibus_config_get_value (config, "engine/Array", "OutputSimplified");
+    if (value && g_variant_classify(value) == G_VARIANT_CLASS_BOOLEAN)
+            is_output_simplified = g_variant_get_boolean(value);
+
+    value = ibus_config_get_value (config, "engine/Array", "UseShift");
+    if (value && g_variant_classify(value) == G_VARIANT_CLASS_BOOLEAN)
+            is_use_shift = g_variant_get_boolean(value);
+
+    value = ibus_config_get_value (config, "engine/Array", "Mode");
+    if (value && g_variant_classify(value) == G_VARIANT_CLASS_BOOLEAN)
+            is_english_mode = g_variant_get_boolean(value);
+
+    value = ibus_config_get_value (config, "engine/Array", "FullWidth");
+    if (value && g_variant_classify(value) == G_VARIANT_CLASS_BOOLEAN)
+            is_fullwidth_mode = g_variant_get_boolean(value);
+
     /* gettext preparation */
     setlocale (LC_ALL, "");
     bindtextdomain (PACKAGE, LOCALEDIR);
     textdomain (PACKAGE);
+
+#ifdef HAVE_OPENCC
+    cc_handle = opencc_open("tw2s.json");
+#endif
 }
 
-void ibus_array_exit (void) 
+void ibus_array_exit (void)
 {
-    array_release_context(array_context);    
+#ifdef HAVE_OPENCC
+    if (cc_handle)
+        opencc_close(cc_handle);
+#endif
+
+    array_release_context(array_context);
 
     if (g_object_is_floating (config))
         g_object_unref(config);
@@ -164,7 +219,7 @@ static void ibus_array_engine_class_init (IBusArrayEngineClass *klass)
     IBusEngineClass *engine_class = IBUS_ENGINE_CLASS (klass);
 
     parent_class = (IBusEngineClass *) g_type_class_peek_parent (klass);
-	
+
     ibus_object_class->destroy = (IBusObjectDestroyFunc) ibus_array_engine_destroy;
 
     engine_class->process_key_event = ibus_array_engine_process_key_event;
@@ -181,6 +236,14 @@ static void ibus_array_engine_class_init (IBusArrayEngineClass *klass)
 
 static void ibus_array_engine_init (IBusArrayEngine *arrayeng)
 {
+    IBusProperty *mode_prop;
+    IBusText *mode_label;
+    IBusText *mode_tooltip;
+
+    IBusProperty *fullwidth_prop;
+    IBusText *fullwidth_label;
+    IBusText *fullwidth_tooltip;
+
     IBusProperty *setup_prop;
     IBusText *setup_label;
     IBusText *setup_tooltip;
@@ -188,9 +251,31 @@ static void ibus_array_engine_init (IBusArrayEngine *arrayeng)
     arrayeng->preedit = g_string_new ("");
     arrayeng->cursor_pos = 0;
     arrayeng->space_press_count = 0;
+    arrayeng->wildcard_char_count = 0;
 
     arrayeng->table = ibus_lookup_table_new (10, 0, FALSE, TRUE);
     g_object_ref_sink (arrayeng->table);
+
+    mode_label = is_english_mode ?
+                 ibus_text_new_from_string (_("English mode")):
+                 ibus_text_new_from_string (_("Chinese mode"));
+    mode_tooltip = is_english_mode ?
+                   ibus_text_new_from_string (_("Switch to Chinese mode")):
+                   ibus_text_new_from_string (_("Switch to English mode"));
+    mode_prop = ibus_property_new("mode", PROP_TYPE_NORMAL, mode_label,
+                                  "gtk-convert", mode_tooltip, TRUE, TRUE, 0, NULL);
+    g_object_ref_sink (mode_prop);
+
+    fullwidth_label = is_fullwidth_mode ?
+                      ibus_text_new_from_string (_("Full width mode")):
+                      ibus_text_new_from_string (_("Half width mode"));
+    fullwidth_tooltip = is_fullwidth_mode ?
+                        ibus_text_new_from_string (_("Switch to Half width mode")):
+                        ibus_text_new_from_string (_("Switch to Full width mode"));
+    fullwidth_prop = ibus_property_new("fullwidth", PROP_TYPE_NORMAL,fullwidth_label,
+                                       "gtk-convert", fullwidth_tooltip, TRUE, TRUE, 0, NULL);
+    g_object_ref_sink (fullwidth_prop);
+
     setup_label = ibus_text_new_from_string (_("Setup"));
     setup_tooltip = ibus_text_new_from_string (_("Configure Array 30 engine"));
     setup_prop = ibus_property_new("setup", PROP_TYPE_NORMAL, setup_label, "gtk-preferences", setup_tooltip, TRUE, TRUE, 0, NULL);
@@ -199,6 +284,8 @@ static void ibus_array_engine_init (IBusArrayEngine *arrayeng)
     arrayeng->prop_list = ibus_prop_list_new();
     g_object_ref_sink (arrayeng->prop_list);
 
+    ibus_prop_list_append(arrayeng->prop_list, mode_prop);
+    ibus_prop_list_append(arrayeng->prop_list, fullwidth_prop);
     ibus_prop_list_append(arrayeng->prop_list, setup_prop);
 
     g_signal_connect (config, "value-changed", G_CALLBACK(ibus_config_value_changed_cb), NULL);
@@ -228,6 +315,7 @@ static void ibus_array_engine_reset(IBusEngine *engine) {
     g_string_assign (arrayeng->preedit, "");
     arrayeng->cursor_pos = 0;
     arrayeng->space_press_count = 0;
+    arrayeng->wildcard_char_count = 0;
 
     ibus_engine_hide_preedit_text (engine);
     ibus_engine_hide_lookup_table (engine);
@@ -266,11 +354,12 @@ static void ibus_array_engine_update_lookup_table (IBusArrayEngine *arrayeng) {
 
     GArray *candidates = NULL;
 
-    if (arrayeng->preedit->len <= 2 && arrayeng->space_press_count == 0)
+    if (arrayeng->preedit->len <= 2 && arrayeng->space_press_count == 0 &&
+        arrayeng->wildcard_char_count == 0)
         candidates = array_get_candidates_from_simple(array_context, arrayeng->preedit->str);
     else
-        candidates = array_get_candidates_from_main(array_context, arrayeng->preedit->str);
-    
+        candidates = array_get_candidates_from_main(array_context, arrayeng->preedit->str, arrayeng->wildcard_char_count);
+
     if (candidates == NULL) {
         ibus_engine_hide_lookup_table ((IBusEngine *) arrayeng);
         return;
@@ -307,16 +396,19 @@ static void ibus_array_engine_update_preedit (IBusArrayEngine *arrayeng) {
         if (retval != 0)
             ibus_attr_list_append (text->attrs, ibus_attr_foreground_new (0xff0000, 0, array_preedit->len));
     }
-    
+
     ibus_engine_update_preedit_text ((IBusEngine *)arrayeng, text, array_preedit->len, TRUE);
 
+#if IBUS_CHECK_VERSION (1, 3, 0)
+#else
     if (G_IS_OBJECT (text) && g_object_is_floating (text))
     	g_object_unref (text);
+#endif
 
     g_string_free(array_preedit, TRUE);
 }
 
-static gboolean ibus_array_engine_update_symbol_lookup_table (IBusArrayEngine *arrayeng) 
+static gboolean ibus_array_engine_update_phrase_lookup_table (IBusArrayEngine *arrayeng)
 {
     gint i;
     gboolean retval;
@@ -330,8 +422,43 @@ static gboolean ibus_array_engine_update_symbol_lookup_table (IBusArrayEngine *a
 
     GArray *candidates = NULL;
 
-    candidates = array_get_candidates_from_main(array_context, arrayeng->preedit->str);
-    
+    candidates = array_get_candidates_from_phrase(array_context, arrayeng->preedit->str);
+
+    if (candidates == NULL) {
+        ibus_engine_hide_lookup_table ((IBusEngine *) arrayeng);
+        return FALSE;
+    }
+    else if (candidates->len == 0) {
+        array_release_candidates(candidates);
+        ibus_engine_hide_lookup_table ((IBusEngine *) arrayeng);
+        return FALSE;
+    }
+
+    for (i = 0; i < candidates->len; i++)
+        ibus_lookup_table_append_candidate (arrayeng->table, ibus_text_new_from_string (g_array_index(candidates, gchar*, i)));
+
+    array_release_candidates(candidates);
+
+    ibus_engine_update_lookup_table ((IBusEngine *) arrayeng, arrayeng->table, TRUE);
+    return TRUE;
+}
+
+static gboolean ibus_array_engine_update_symbol_lookup_table (IBusArrayEngine *arrayeng)
+{
+    gint i;
+    gboolean retval;
+
+    if (arrayeng->preedit->len == 0) {
+        ibus_engine_hide_lookup_table ((IBusEngine *) arrayeng);
+        return FALSE;
+    }
+
+    ibus_lookup_table_clear (arrayeng->table);
+
+    GArray *candidates = NULL;
+
+    candidates = array_get_candidates_from_main(array_context, arrayeng->preedit->str, arrayeng->wildcard_char_count);
+
     if (candidates == NULL) {
         ibus_engine_hide_lookup_table ((IBusEngine *) arrayeng);
         return FALSE;
@@ -375,8 +502,26 @@ static gboolean ibus_array_engine_commit_current_candidate (IBusArrayEngine *arr
                     return FALSE;
             }
         }
-        ibus_engine_commit_text((IBusEngine*)arrayeng, text);
-        
+
+#ifdef HAVE_OPENCC
+        if (is_output_simplified && cc_handle != NULL) {
+            char *converted = opencc_convert_utf8(cc_handle,
+                                                  text->text,
+                                                  strlen(text->text));
+            if(converted) {
+                IBusText* newtext = ibus_text_new_from_string(converted);
+                ibus_engine_commit_text((IBusEngine*)arrayeng, newtext);
+                opencc_convert_utf8_free(converted);
+            } else {
+                ibus_engine_commit_text((IBusEngine*)arrayeng, text);
+            }
+        } else {
+#endif
+            ibus_engine_commit_text((IBusEngine*)arrayeng, text);
+#ifdef HAVE_OPENCC
+        }
+#endif
+
         ibus_array_engine_reset((IBusEngine*)arrayeng);
 
         if (is_special_notify && check_special)
@@ -396,8 +541,11 @@ static void ibus_array_engine_commit_string (IBusArrayEngine *arrayeng, const gc
     IBusText *text;
     text = ibus_text_new_from_static_string (string);
     ibus_engine_commit_text ((IBusEngine *)arrayeng, text);
+#if IBUS_CHECK_VERSION (1, 3, 0)
+#else
     if (g_object_is_floating (text))
     	g_object_unref (text);
+#endif
 }
 
 static void ibus_array_engine_update (IBusArrayEngine *arrayeng) {
@@ -407,6 +555,8 @@ static void ibus_array_engine_update (IBusArrayEngine *arrayeng) {
 }
 
 #define is_alpha(c) (((c) >= IBUS_a && (c) <= IBUS_z))
+#define is_root(c)  ((is_alpha (c)  || (c) == IBUS_period || (c) == IBUS_comma || (c) == IBUS_slash || (c) == IBUS_semicolon))
+#define is_wildcard(c) (((c) == IBUS_question))
 
 static gboolean  ibus_array_engine_process_key_event (IBusEngine *engine, guint keyval, guint keycode, guint modifiers) {
     IBusText *text;
@@ -422,8 +572,35 @@ static gboolean  ibus_array_engine_process_key_event (IBusEngine *engine, guint
         is_aux_shown = FALSE;
     }
 
-    if (modifiers & IBUS_RELEASE_MASK)
+    if (modifiers & IBUS_RELEASE_MASK) {
+        gboolean triggered = FALSE;
+
+        /* The keyval must be the same */
+        if (prev_pressed_key == keyval) {
+            if (keyval == IBUS_Shift_L || keyval == IBUS_Shift_R)
+                triggered = TRUE;
+        }
+
+        if (triggered) {
+            if (is_use_shift) {
+                ibus_array_engine_property_activate((IBusEngine *) engine,
+                                                    "mode",
+                                                    PROP_STATE_UNCHECKED);
+                return TRUE;
+            }
+        }
+
         return FALSE;
+    }
+
+    prev_pressed_key = keyval;
+
+    if ((modifiers & IBUS_SHIFT_MASK) && keyval == IBUS_space) {
+        ibus_array_engine_property_activate((IBusEngine *) engine,
+                                            "fullwidth",
+                                            PROP_STATE_UNCHECKED);
+        return TRUE;
+    }
 
     if (keyval == IBUS_Shift_L || keyval == IBUS_Shift_R)
         return FALSE;
@@ -431,14 +608,65 @@ static gboolean  ibus_array_engine_process_key_event (IBusEngine *engine, guint
     if (modifiers & (IBUS_CONTROL_MASK | IBUS_MOD1_MASK))
         return FALSE;
 
+    if (is_english_mode) {
+        if (is_fullwidth_mode) {
+            if (keyval >= 32 && keyval - 32 < sizeof(sFullWidthTable)) {
+                IBusText* newtext = ibus_text_new_from_string(sFullWidthTable[keyval - 32]);
+                ibus_engine_commit_text((IBusEngine*)arrayeng, newtext);
+                return TRUE;
+            }
+        }
+
+        return FALSE;
+    }
 
     switch (keyval) {
     case IBUS_space:
-        if (arrayeng->preedit->len == 0)
+        if (arrayeng->preedit->len == 0) {
+            if (is_fullwidth_mode) {
+                if (keyval >= 32 && keyval - 32 < sizeof(sFullWidthTable)) {
+                    IBusText* newtext = ibus_text_new_from_string(sFullWidthTable[keyval - 32]);
+                    ibus_engine_commit_text((IBusEngine*)arrayeng, newtext);
+                    return TRUE;
+                }
+            }
+
             return FALSE;
+        }
+
         ibus_array_engine_space_press(arrayeng);
         return TRUE;
 
+    case IBUS_apostrophe:
+        if (arrayeng->preedit->len == 0) {
+            if (is_fullwidth_mode) {
+                if (keyval >= 32 && keyval - 32 < sizeof(sFullWidthTable)) {
+                    IBusText* newtext = ibus_text_new_from_string(sFullWidthTable[keyval - 32]);
+                    ibus_engine_commit_text((IBusEngine*)arrayeng, newtext);
+                    return TRUE;
+                }
+            }
+
+            return FALSE;
+        }
+
+        /* The behavior of phrase key is like space key first time */
+        if (arrayeng->space_press_count == 0) {
+            arrayeng->space_press_count ++;
+        }
+
+        ibus_array_engine_update_phrase_lookup_table (arrayeng);
+
+        if (arrayeng->table->candidates->len == 1) {
+            gboolean commit_rev;
+
+            ibus_lookup_table_set_cursor_pos(arrayeng->table, 0);
+            commit_rev = ibus_array_engine_commit_current_candidate(arrayeng);
+        } else if (arrayeng->table->candidates->len == 0) {
+            ibus_array_engine_reset((IBusEngine*)arrayeng);
+        }
+        return TRUE;
+
     case IBUS_Return:
         if (arrayeng->preedit->len == 0)
             return FALSE;
@@ -446,8 +674,10 @@ static gboolean  ibus_array_engine_process_key_event (IBusEngine *engine, guint
             return TRUE;
 
     case IBUS_Escape:
+        if (arrayeng->preedit->len == 0)
+            return FALSE;
         ibus_array_engine_reset((IBusEngine*)arrayeng);
-        return TRUE;        
+        return TRUE;
 
     case IBUS_Left:
     case IBUS_Up:
@@ -473,10 +703,13 @@ static gboolean  ibus_array_engine_process_key_event (IBusEngine *engine, guint
             return FALSE;
         if (arrayeng->cursor_pos > 0) {
             arrayeng->cursor_pos-- ;
+            if (is_wildcard (arrayeng->preedit->str[arrayeng->cursor_pos]))
+                arrayeng->wildcard_char_count --;
+            arrayeng->space_press_count = 0;
             g_string_erase (arrayeng->preedit, arrayeng->cursor_pos, 1);
             ibus_array_engine_update (arrayeng);
         }
-        return TRUE; 
+        return TRUE;
     }
 
     if (arrayeng->preedit->len > 0 && (keyval >= IBUS_0 && keyval <= IBUS_9)) {
@@ -491,7 +724,7 @@ static gboolean  ibus_array_engine_process_key_event (IBusEngine *engine, guint
         return ibus_array_engine_process_candidate_key_event(arrayeng, keyval, modifiers);
     }
 
-    if (is_alpha (keyval) || keyval == IBUS_period || keyval == IBUS_comma || keyval == IBUS_slash || keyval == IBUS_semicolon) {
+    if (is_root (keyval) || is_wildcard (keyval)) {
         if (arrayeng->space_press_count == 1)
             if (arrayeng->table->candidates->len > 0) {
                 gboolean commit_rev;
@@ -500,20 +733,31 @@ static gboolean  ibus_array_engine_process_key_event (IBusEngine *engine, guint
 
                 commit_rev = ibus_array_engine_commit_current_candidate(arrayeng);
             } else {
-		ibus_array_engine_reset((IBusEngine*)arrayeng);
+                ibus_array_engine_reset((IBusEngine*)arrayeng);
             }
 
         if (arrayeng->preedit->len >= 5)
                 return TRUE;
 
+        if (is_wildcard (keyval))
+            arrayeng->wildcard_char_count ++;
+
         g_string_insert_c (arrayeng->preedit, arrayeng->cursor_pos, keyval);
 
         arrayeng->cursor_pos ++;
         ibus_array_engine_update (arrayeng);
-        
+
         return TRUE;
     }
 
+    if (is_fullwidth_mode) {
+        if (keyval >= 32 && keyval - 32 < sizeof(sFullWidthTable)) {
+            IBusText* newtext = ibus_text_new_from_string(sFullWidthTable[keyval - 32]);
+            ibus_engine_commit_text((IBusEngine*)arrayeng, newtext);
+            return TRUE;
+        }
+    }
+
     return FALSE;
 }
 
@@ -533,7 +777,7 @@ static gboolean ibus_array_engine_process_candidate_key_event (IBusArrayEngine *
         else
             cursor_pos = page_no * page_size + 9;
 
-        if (cursor_pos >= arrayeng->table->candidates->len) 
+        if (cursor_pos >= arrayeng->table->candidates->len)
             return TRUE;
 
         ibus_lookup_table_set_cursor_pos (arrayeng->table, cursor_pos);
@@ -553,7 +797,7 @@ static void ibus_array_engine_space_press (IBusArrayEngine *arrayeng)
         ibus_engine_update_lookup_table ((IBusEngine *) arrayeng, arrayeng->table, TRUE);
         return;
     }
-    
+
     if (arrayeng->space_press_count == 0) {
         arrayeng->space_press_count ++;
 
@@ -564,6 +808,8 @@ static void ibus_array_engine_space_press (IBusArrayEngine *arrayeng)
 
             ibus_lookup_table_set_cursor_pos(arrayeng->table, 0);
             commit_rev = ibus_array_engine_commit_current_candidate(arrayeng);
+        } else if (arrayeng->table->candidates->len == 0) {
+            ibus_array_engine_reset((IBusEngine*)arrayeng);
         }
     }
     else if (arrayeng->space_press_count == 1) {
@@ -639,7 +885,9 @@ static void ibus_array_engine_show_special_code_for_char (IBusArrayEngine *array
     array_release_candidates(candidates);
 }
 
-static void ibus_array_engine_property_activate (IBusEngine *engine, const gchar *prop_name, gint prop_state) {
+static void ibus_array_engine_property_activate (IBusEngine *engine, const gchar *prop_name, guint prop_state) {
+    IBusArrayEngine *arrayeng = (IBusArrayEngine*)engine;
+
     if (g_strcmp0(prop_name, "setup") == 0) {
         GError *error = NULL;
         gchar *argv[2] = { NULL, };
@@ -656,6 +904,56 @@ static void ibus_array_engine_property_activate (IBusEngine *engine, const gchar
         g_spawn_async (NULL, argv, NULL, 0, NULL, NULL, NULL, &error);
 
         g_free(path);
+    } else if (g_strcmp0(prop_name, "mode") == 0) {
+        int i = 0;
+        IBusProperty *prop;
+        while ((prop = ibus_prop_list_get (arrayeng->prop_list, i)) != NULL) {
+            if(g_strcmp0(ibus_property_get_key(prop), prop_name) == 0) {
+                is_english_mode = !is_english_mode;
+
+                ibus_property_set_label (prop, is_english_mode?
+                            ibus_text_new_from_string (_("English mode")):
+                            ibus_text_new_from_string (_("Chinese mode")));
+                ibus_property_set_tooltip (prop, is_english_mode?
+                            ibus_text_new_from_string (_("Switch to Chinese mode")):
+                            ibus_text_new_from_string (_("Switch to English mode")));
+
+                ibus_engine_update_property(engine, prop);
+
+                ibus_config_set_value (config, "engine/Array", "Mode",
+                                       g_variant_new_boolean(is_english_mode));
+
+                ibus_array_engine_reset((IBusEngine*)arrayeng);
+                break;
+            }
+
+            i++;
+        }
+    } else if (g_strcmp0(prop_name, "fullwidth") == 0) {
+        int i = 0;
+        IBusProperty *prop;
+        while ((prop = ibus_prop_list_get (arrayeng->prop_list, i)) != NULL) {
+            if(g_strcmp0(ibus_property_get_key(prop), prop_name) == 0) {
+                is_fullwidth_mode = !is_fullwidth_mode;
+
+                ibus_property_set_label (prop, is_fullwidth_mode?
+                            ibus_text_new_from_string (_("Full width mode")):
+                            ibus_text_new_from_string (_("Half width mode")));
+                ibus_property_set_tooltip (prop, is_fullwidth_mode?
+                            ibus_text_new_from_string (_("Switch to Half width mode")):
+                            ibus_text_new_from_string (_("Switch to Full width mode")));
+
+                ibus_engine_update_property(engine, prop);
+
+                ibus_config_set_value (config, "engine/Array", "FullWidth",
+                                       g_variant_new_boolean(is_fullwidth_mode));
+
+                ibus_array_engine_reset((IBusEngine*)arrayeng);
+                break;
+            }
+
+            i++;
+        }
     }
 }
 
@@ -665,5 +963,9 @@ static void ibus_config_value_changed_cb (IBusConfig *config, const gchar *secti
             is_special_notify = g_variant_get_boolean (value);
         else if (g_strcmp0(name, "specialonly") == 0)
             is_special_only = g_variant_get_boolean (value);
+        else if (g_strcmp0(name, "outputsimplified") == 0)
+            is_output_simplified = g_variant_get_boolean (value);
+        else if (g_strcmp0(name, "useshift") == 0)
+            is_use_shift = g_variant_get_boolean (value);
 }
 
